﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using DocumentFormat.OpenXml.Packaging;
using DocumentFormat.OpenXml.Wordprocessing;
using ss = DocumentFormat.OpenXml.Spreadsheet;
using dr = DocumentFormat.OpenXml.Drawing;
using DocumentFormat.OpenXml.Drawing.Charts;

class Program
{
    static int[] newData = { 6, 7, 6 };

    public static void CopyStream(Stream input, Stream output)
    {
        byte[] buffer = new byte[32768];
        while (true)
        {
            int read = input.Read(buffer, 0, buffer.Length);
            if (read <= 0)
                return;
            output.Write(buffer, 0, read);
        }
    }
    
    static void Main(string[] args)
    {
        using (WordprocessingDocument doc =
            WordprocessingDocument.Open("Embedded.docx", true))
        {
            string rid = null;

            // Find chart (in this case, it is in the first paragraph).
            // Then get the relationship id.
            Paragraph p = doc
                .MainDocumentPart
                .Document
                .Body
                .Elements<Paragraph>()
                .FirstOrDefault();
            if (p != null)
            {
                Drawing d = p.Descendants<Drawing>().FirstOrDefault();
                if (d != null)
                {
                    ChartReference cr = d
                        .Descendants<ChartReference>()
                        .FirstOrDefault();
                    if (cr != null)
                        rid = cr.Id;
                }
            }
            ChartPart cp = (ChartPart)doc
                .MainDocumentPart
                .Parts
                .Where(pt => pt.RelationshipId == rid)
                .FirstOrDefault()
                .OpenXmlPart;
            string externalDataRid = null;
            if (cp != null)
            {
                ExternalData ed = cp
                    .ChartSpace
                    .Elements<ExternalData>()
                    .FirstOrDefault();
                if (ed != null)
                    externalDataRid = ed.Id;
            }
            EmbeddedPackagePart epp = (EmbeddedPackagePart)cp
                .Parts
                .Where(pt => pt.RelationshipId == externalDataRid)
                .FirstOrDefault()
                .OpenXmlPart;
            using (Stream str = epp.GetStream())
            using (MemoryStream ms = new MemoryStream())
            {
                CopyStream(str, ms);
                using (SpreadsheetDocument spreadsheetDoc =
                    SpreadsheetDocument.Open(ms, true))
                {
                    // Update data in spreadsheet
                    // Find first worksheet
                    ss.Sheet ws = (ss.Sheet)spreadsheetDoc.WorkbookPart
                        .Workbook.Sheets.FirstOrDefault();
                    string sheetId = ws.Id;

                    WorksheetPart wsp = (WorksheetPart)spreadsheetDoc
                        .WorkbookPart
                        .Parts
                        .Where(pt => pt.RelationshipId == sheetId)
                        .FirstOrDefault()
                        .OpenXmlPart;
                    ss.SheetData sd = wsp
                        .Worksheet
                        .Elements<ss.SheetData>()
                        .FirstOrDefault();
                    ss.Row secondRow = sd
                        .Elements<ss.Row>()
                        .Skip(1)
                        .FirstOrDefault();
                    if (secondRow != null)
                    {
                        ss.CellValue cell1value = secondRow
                            .Elements<ss.Cell>()
                            .ElementAt(1)
                            .Elements<ss.CellValue>()
                            .FirstOrDefault();
                        cell1value.Text = newData[0].ToString();
                        ss.CellValue cell2value = secondRow
                            .Elements<ss.Cell>()
                            .ElementAt(2)
                            .Elements<ss.CellValue>()
                            .FirstOrDefault();
                        cell2value.Text = newData[1].ToString();
                        ss.CellValue cell3value = secondRow
                            .Elements<ss.Cell>()
                            .ElementAt(3)
                            .Elements<ss.CellValue>()
                            .FirstOrDefault();
                        cell3value.Text = newData[2].ToString();
                    }
                }
                // Write the modified memory stream back
                // into the embedded package part.
                using (Stream s = epp.GetStream())
                    ms.WriteTo(s);
            }
            Chart chart = cp.ChartSpace.Elements<Chart>().First();
            BarChart bc = chart
                .Descendants<BarChart>()
                .FirstOrDefault();
            if (bc != null)
            {
                BarChartSeries bcs = bc
                    .Elements<BarChartSeries>()
                    .FirstOrDefault();
                if (bcs != null)
                {
                    Values values = bcs
                        .Descendants<Values>()
                        .First();
                    NumberingCache nc = values
                        .Descendants<NumberingCache>()
                        .First();
                    NumericValue nv1 = nc
                        .Elements<NumericPoint>()
                        .ElementAt(0)
                        .Elements<NumericValue>()
                        .FirstOrDefault();
                    nv1.Text = newData[0].ToString();
                    NumericValue nv2 = nc
                        .Elements<NumericPoint>()
                        .ElementAt(1)
                        .Elements<NumericValue>()
                        .FirstOrDefault();
                    nv2.Text = newData[1]
                        .ToString();
                    NumericValue nv3 = nc
                        .Elements<NumericPoint>()
                        .ElementAt(2)
                        .Elements<NumericValue>()
                        .FirstOrDefault();
                    nv3.Text = newData[2].ToString();
                }
            }
        }
    }
}
